using System;
using System.IO;
using Server;

namespace Server.Scripts.Commands
{
	public class CommandLogging
	{
		private static StreamWriter m_Output;

		public static StreamWriter Output{ get{ return m_Output; } }

		public static void Initialize()
		{
			EventSink.Command += new CommandEventHandler( EventSink_Command );

			if ( !Directory.Exists( "Logs" ) )
				Directory.CreateDirectory( "Logs" );

			string directory = String.Format( "Logs/{0} {1}", DateTime.Now.DayOfYear, DateTime.Now.DayOfWeek );

			if ( !Directory.Exists( directory ) )
				Directory.CreateDirectory( directory );

			m_Output = new StreamWriter( Path.Combine( directory, "Commands.log" ), true );

			m_Output.AutoFlush = true;

			m_Output.WriteLine( "##############################" );
			m_Output.WriteLine( "Log started on {0}", DateTime.Now );
			m_Output.WriteLine();
		}

		public static string FormatMobile( Mobile m )
		{
			if ( m.Account == null )
				return String.Format( "{0} (no account)", m );
			else
				return String.Format( "{0} ('{1}')", m, m.Account.Username );
		}

		public static void EventSink_Command( CommandEventArgs e )
		{
			m_Output.WriteLine( "{0}: {1}: {2} {3} used command '{4} {5}'", DateTime.Now, e.Mobile.NetState, e.Mobile.AccessLevel, FormatMobile( e.Mobile ), e.Command, e.ArgString );
		}

		public static void LogChangeProperty( Mobile from, object o, string name, string value )
		{
			if ( o is Mobile )
				m_Output.WriteLine( "{0}: {1}: {2} {3} set property '{4}' of {5} to '{6}'", DateTime.Now, from.NetState, from.AccessLevel, FormatMobile( from ), name, FormatMobile( o as Mobile ), value );
			else
				m_Output.WriteLine( "{0}: {1}: {2} {3} set property '{4}' of {5} to '{6}'", DateTime.Now, from.NetState, from.AccessLevel, FormatMobile( from ), name, o, value );
		}
	}
}