using System;
using System.Collections;
using Server.Network;

namespace Server.Gumps
{
	public class WhoGump : Gump
	{
		public static void Initialize()
		{
			Commands.Register( "Admin", AccessLevel.Counselor, new CommandEventHandler( WhoList_OnCommand ) );
			Commands.Register( "Who", AccessLevel.Counselor, new CommandEventHandler( WhoList_OnCommand ) );
			Commands.Register( "WhoList", AccessLevel.Counselor, new CommandEventHandler( WhoList_OnCommand ) );
		}

		[Usage( "WhoList" )]
		[Aliases( "Who", "Admin" )]
		[Description( "Lists all connected clients." )]
		private static void WhoList_OnCommand( CommandEventArgs e )
		{
			e.Mobile.SendGump( new WhoGump( e.Mobile ) );
		}

		private Mobile m_Owner;
		private ArrayList m_Mobiles;

		private class InternalComparer : IComparer
		{
			public static readonly IComparer Instance = new InternalComparer();

			public InternalComparer()
			{
			}

			public int Compare( object x, object y )
			{
				if ( x == null && y == null )
					return 0;
				else if ( x == null )
					return -1;
				else if ( y == null )
					return 1;

				Mobile a = x as Mobile;
				Mobile b = y as Mobile;

				if ( a == null || b == null )
					throw new ArgumentException();

				if ( a.AccessLevel > b.AccessLevel )
					return -1;
				else if ( a.AccessLevel < b.AccessLevel )
					return 1;
				else
					return CaseInsensitiveComparer.Default.Compare( a.Name, b.Name );
			}
		}

		public WhoGump( Mobile owner ) : base( 30, 20 )
		{
			m_Owner = owner;
			m_Mobiles = new ArrayList();

			ArrayList states = NetState.Instances;

			for ( int i = 0; i < states.Count; ++i )
			{
				Mobile m = ((NetState)states[i]).Mobile;

				if ( m != null && (m == owner || !m.Hidden || owner.AccessLevel > m.AccessLevel) )
					m_Mobiles.Add( m );
			}

			m_Mobiles.Sort( InternalComparer.Instance );

			AddPage( 0 );

			AddBackground( 0, 0, 260, 351, 5054 );

			AddImageTiled( 10, 10, 240, 23, 0x52 );
			AddImageTiled( 11, 11, 238, 21, 0xBBC );

			for ( int i = 0; i < m_Mobiles.Count && i < 14; ++i )
			{
				AddImageTiled( 10, 32 + (i * 22), 240, 23, 0x52 );
				AddImageTiled( 11, 33 + (i * 22), 238, 21, 0xBBC );
			}

			for ( int i = 0; i < m_Mobiles.Count; ++i )
			{
				Mobile m = (Mobile)m_Mobiles[i];

				if ( (i % 14) == 0 )
				{
					if ( i != 0 )
					{
						AddButton( 230, 13, 0x15E1, 0x15E5, 0, GumpButtonType.Page, 1 + (i / 14) );
						AddLabel( 194, 11, 0, "Next" );
					}

					AddPage( 1 + (i / 14) );

					if ( m_Mobiles.Count > 14 && (m_Mobiles.Count / 14) == (i / 14) )
					{
						// Last page, rebuild background
						AddBackground( 0, 0, 260, 351, 5054 );

						AddImageTiled( 10, 10, 240, 23, 0x52 );
						AddImageTiled( 11, 11, 238, 21, 0xBBC );

						for ( int j = 0; j < (m_Mobiles.Count % 14); ++j )
						{
							AddImageTiled( 10, 32 + (j * 22), 240, 23, 0x52 );
							AddImageTiled( 11, 33 + (j * 22), 238, 21, 0xBBC );
						}
					}

					if ( i != 0 )
					{
						AddButton( 13, 13, 0x15E3, 0x15E7, 0, GumpButtonType.Page, i / 14 );
						AddLabel( 33, 11, 0, "Previous" );
					}
				}

				AddLabelCropped( 14, 33 + ((i % 14) * 22), 195, 21, GetHueFor( m ), m.Name );
				AddButton( 230, 35 + ((i % 14) * 22), 0x15E1, 0x15E5, 1 + i, GumpButtonType.Reply, 0 );
			}
		}

		public override void OnResponse(NetState sender, RelayInfo info)
		{
			if ( info.ButtonID >= 1 && info.ButtonID <= m_Mobiles.Count )
			{
				Mobile m = (Mobile)m_Mobiles[info.ButtonID - 1];

				if ( m != null && m.NetState != null )
					sender.Mobile.SendGump( new ClientGump( sender.Mobile, m.NetState ) );
			}
		}

		private static int GetHueFor( Mobile m )
		{
			switch ( m.AccessLevel )
			{
				case AccessLevel.Player:
				{
					if ( m.Kills >= 5 )
					{
						return 0x21;
					}
					else if ( m.Criminal )
					{
						return 0x3B1;
					}

					return 0x58;
				}
				case AccessLevel.Counselor: return 0x2;
				case AccessLevel.GameMaster: return 0x21;
				case AccessLevel.Seer: return 0x144;
				case AccessLevel.Administrator: return 0x516;
				default: return 0;
			}
		}
	}
}