using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public class DefCarpentry : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Carpentry;	}
		}

		public override string GumpTitle
		{
			get { return "CARPENTRY MENU"; }
		}

		public override int GumpTitlePos
		{
			get { return 175; }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefCarpentry();

				return m_CraftSystem;
			}
		}

		private DefCarpentry() : base( 1, 1, 3.0 )
		{
		}

		public override bool CanCraft( Mobile from, BaseTool tool, Type itemType )
		{
			if ( tool.Deleted )
			{
				return false;
			}
			else
			{
				if ( tool.Durability < 1 )
				{
					from.SendMessage( "This tool is broken" );
					return false;
				}
			}

			return true;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			if ( from.Body.Type == BodyType.Human && !from.Mounted )
				from.Animate( 9, 5, 1, true, false, 0 );

			from.PlaySound( 0x23D );
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality )
		{
			if ( toolBroken )
			{
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool
			}

			if ( failed )
			{
				if ( lostMaterial) 
				{
					from.SendLocalizedMessage( 1044043 ); // You failed to create the item, and some of your materials are lost.
				}
				else
				{
					from.SendLocalizedMessage( 1044157 ); // You failed to create the item, but no materials were lost.
				}
			}
			else
			{
				if ( quality == 0 )
					from.SendLocalizedMessage( 502785 ); // You were barely able to make the item.  It's quality is below average
				else if (quality == 2 )
					from.SendLocalizedMessage( 502784 ); // Due to your exceptional skill, its quality is higher than average
				else				
					from.SendLocalizedMessage( 1044154 ); // You create the item
			}
		}

		public override void InitCraftList()
		{
			int index = -1;
			
			// Other Items
			index = AddCraft( typeof( Board ), "Other Items", "Board", 0.0, 20.0, typeof( Log ), "Log", 1 );
			SetUseAllRes( index, true );

			AddCraft( typeof( BarrelStaves ), "Other Items", "Barrel Staves", 0.0, 20.0, typeof( Log ), "Log", 5 );
			AddCraft( typeof( BarrelLid ), "Other Items", "Barrel Lid", 11.0, 35.0, typeof( Log ), "Log", 4 );
			AddCraft( typeof( ShortMusicStand ), "Other Items", "Short Music Stand", 78.9, 100.0, typeof( Log ), "Log", 15 );
			AddCraft( typeof( TallMusicStand ), "Other Items", "Tall Music Stand", 81.5, 100.0, typeof( Log ), "Log", 20 );
			AddCraft( typeof( Easle ), "Other Items", "Easle", 86.8, 100.0, typeof( Log ), "Log", 20 );

			// Furniture
			AddCraft( typeof( FootStool ), "Furniture", "Foot Stool", 11.0, 30.0, typeof( Log ), "Log", 9 );
			AddCraft( typeof( Stool ), "Furniture", "Stool", 11.0, 30.0, typeof( Log ), "Log", 9 );
			
			index = AddCraft( typeof( FancyWoodenChairCushion ), "Furniture", "Fancy Wooden Chair Cushion", 55.0, 70.0, typeof( Log ), "Log", 15 );
			AddSkill( index, SkillName.Tailoring, 20.0, 40.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 5 );
			
			index = AddCraft( typeof( WoodenChairCushion ), "Furniture", "Wooden Chair Cushion", 21.0, 40.0, typeof( Log ), "Log", 13 );
			AddSkill( index, SkillName.Tailoring, 15.0, 30.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 5 );
			
			AddCraft( typeof( WoodenChair ), "Furniture", "Wooden Chair", 21.0, 40.0, typeof( Log ), "Log", 13 );
			AddCraft( typeof( BambooChair ), "Furniture", "Bamboo Chair", 21.0, 40.0, typeof( Log ), "Log", 13 );
			AddCraft( typeof( WoodenBench ), "Furniture", "Wooden Bench", 52.6, 75.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( Throne ), "Furniture", "Throne", 73.6, 100.0, typeof( Log ), "Log", 19 );
			AddCraft( typeof( WoodenThrone ), "Furniture", "Wooden Throne", 52.6, 75.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( Nightstand ), "Furniture", "Nightstand", 42.1, 65.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( LargeTable ), "Furniture", "Large Table", 63.1, 85.0, typeof( Log ), "Log", 23 );
			AddCraft( typeof( WritingTable ), "Furniture", "Writing Table", 63.1, 85.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( YewWoodTable ), "Furniture", "Yew-Wood Table", 84.2, 100.0, typeof( Log ), "Log", 27 );

			//Containers
			AddCraft( typeof( WoodenBox ), "Containers", "Wooden Box", 21.0, 40.0, typeof(Log ), "Log", 10 );
			AddCraft( typeof( SmallCrate ), "Containers", "Small Crate", 10.0, 30.0, typeof(Log ), "Log", 8 );
			AddCraft( typeof( MediumCrate ), "Containers", "Medium Crate", 31.0, 45.0, typeof(Log ), "Log", 15 );
			AddCraft( typeof( LargeCrate ), "Containers", "Large Crate", 47.0, 52.0, typeof(Log ), "Log", 18 );
			AddCraft( typeof( WoodenChest ), "Containers", "Wooden Chest", 73.6, 85.0, typeof(Log ), "Log", 20 );
			AddCraft( typeof( EmptyBookcase ), "Containers", "Wooden Shelf", 31.5, 45.0, typeof(Log ), "Log", 25 );
			AddCraft( typeof( FancyArmoire ), "Containers", "Armoire (Red)", 84.2, 95.0, typeof(Log ), "Log", 35 );
			AddCraft( typeof( Armoire ), "Containers", "Armoire", 84.2, 95.0, typeof(Log ), "Log", 35 );	

			index = AddCraft( typeof( Keg ), "Containers", "Keg", 57.8, 75.0, typeof( BarrelStaves ), "Barrel Staves", 3 );
			AddRes( index, typeof( BarrelHoops ), "Barrel Hoops", 1 );
			AddRes( index, typeof( BarrelLid ), "Barrel Lid", 1 );


			//Staves and Shields
			AddCraft( typeof( ShepherdsCrook ), "Staves and Shields", "Shepherd's Crook", 78.9, 100.0, typeof( Log ), "Log", 7 );
			AddCraft( typeof( QuarterStaff ), "Staves and Shields", "Quarter Staff", 73.6, 100.0, typeof( Log ), "Log", 6 );
			AddCraft( typeof( GnarledStaff ), "Staves and Shields", "Gnarled Staff", 78.9, 100.0, typeof( Log ), "Log", 7 );
			AddCraft( typeof( BlackStaff ), "Staves and Shields", "Black Staff", 85.9, 100.0, typeof( Log ), "Log", 7 );
			AddCraft( typeof( WoodenShield ), "Staves and Shields", "Wooden Shield", 52.6, 85.0, typeof( Log ), "Log", 9 );
			index = AddCraft( typeof( FishingPole ), "Staves and Shields", "Fishing Pole", 68.4, 100.0, typeof( Log ), "Log", 5 );
			AddSkill( index, SkillName.Tailoring, 40.0, 45.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 5 );

			// Instruments
			index = AddCraft( typeof( LapHarp ), "Instruments", "Lap Harp", 63.1, 100.0, typeof( Log ), "Log", 20 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );

			index = AddCraft( typeof( Harp ), "Instruments", "Standing Harp", 78.9, 100.0, typeof( Log ), "Log", 35 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 15 );
			
			index = AddCraft( typeof( Drums ), "Instruments", "Drum", 57.8, 100.0, typeof( Log ), "Log", 20 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );
			
			index = AddCraft( typeof( Lute ), "Instruments", "Lute", 68.4, 100.0, typeof( Log ), "Log", 25 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );
			
			index = AddCraft( typeof( Tambourine ), "Instruments", "Tambourine", 57.8, 100.0, typeof( Log ), "Log", 15 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );

			index = AddCraft( typeof( TambourineTassel ), "Instruments", "Tambourine (tassel)", 57.8, 100.0, typeof( Log ), "Log", 15 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 15 );	

			// Misc
			index = AddCraft( typeof( SmallBedEastDeed ), "Misc", "Small Bed (East)", 94.7, 110.0, typeof( Log ), "Log", 100 );
			AddSkill( index, SkillName.Tailoring, 75.0, 80.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 100 );
			index = AddCraft( typeof( SmallBedSouthDeed ), "Misc", "Small Bed (South)", 94.7, 110.0, typeof( Log ), "Log", 100 );
			AddSkill( index, SkillName.Tailoring, 75.0, 80.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 100 );
			index = AddCraft( typeof( LargeBedEastDeed ), "Misc", "Large Bed (East)", 94.7, 110.0, typeof( Log ), "Log", 150 );
			AddSkill( index, SkillName.Tailoring, 75.0, 80.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 150 );
			index = AddCraft( typeof( LargeBedSouthDeed ), "Misc", "Large Bed (South)", 94.7, 110.0, typeof( Log ), "Log", 150 );
			AddSkill( index, SkillName.Tailoring, 75.0, 80.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 150 );
			// TODO: I thought this was a deed..
			//AddCraft( typeof( Dartboard ), "Misc", "Dartboard", 15.7, 30.0, typeof( Log ), "Log", 5 );
			index = AddCraft( typeof( PentagramDeed ), "Misc", "Pentagram", 100.0, 120.0, typeof( Log ), "Log", 100);
			AddSkill( index, SkillName.Magery, 75.0, 80.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 40 );
			index = AddCraft( typeof( AbbatoirDeed ), "Misc", "Abbatoir", 100.0, 120.0, typeof( Log ), "Log", 100 );
			AddSkill( index, SkillName.Magery, 50.0, 55.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 40 );

			// Blacksmithy
			index = AddCraft( typeof( SmallForgeDeed ), "Blacksmithy", "Small Forge", 73.6, 90.0, typeof( Log ), "Log", 5 );
			AddSkill( index, SkillName.Blacksmith, 75.0, 80.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 75 );
			index = AddCraft( typeof( LargeForgeEastDeed ), "Blacksmithy", "Large Forge (East)", 78.9, 93.0, typeof( Log ), "Log", 5 );
			AddSkill( index, SkillName.Blacksmith, 80.0, 85.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 100 );
			index = AddCraft( typeof( LargeForgeSouthDeed ), "Blacksmithy", "Large Forge (South)", 78.9, 93.0, typeof( Log ), "Log", 5 );
			AddSkill( index, SkillName.Blacksmith, 80.0, 85.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 100 );
			index = AddCraft( typeof( AnvilEastDeed ), "Blacksmithy", "Anvil (East)", 73.6, 90.0, typeof( Log ), "Log", 5 );
			AddSkill( index, SkillName.Blacksmith, 75.0, 80.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 150 );
			index = AddCraft( typeof( AnvilSouthDeed ), "Blacksmithy", "Anvil (South)", 73.6, 90.0, typeof( Log ), "Log", 5 );
			AddSkill( index, SkillName.Blacksmith, 75.0, 80.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 150 );

			// Tailoring
			index = AddCraft( typeof( Dressform ), "Tailoring", "Dressform", 63.0, 75.0, typeof( Log ), "Log", 25 );
			AddSkill( index, SkillName.Tailoring, 65.0, 70.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );
			index = AddCraft( typeof( SpinningwheelEastDeed ), "Tailoring", "Spinningwheel (East)", 73.6, 85.0, typeof( Log ), "Log", 75 );
			AddSkill( index, SkillName.Tailoring, 65.0, 70.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 25 );
			index = AddCraft( typeof( SpinningwheelSouthDeed ), "Tailoring", "Spinningwheel (South)", 73.6, 85.0, typeof( Log ), "Log", 75 );
			AddSkill( index, SkillName.Tailoring, 65.0, 70.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 25 );
			index = AddCraft( typeof( LoomEastDeed ), "Tailoring", "Loom (East)", 84.2, 98.0, typeof( Log ), "Log", 85 );
			AddSkill( index, SkillName.Tailoring, 65.0, 70.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 25 );
			index = AddCraft( typeof( LoomSouthDeed ), "Tailoring", "Loom (South)", 84.2, 98.0, typeof( Log ), "Log", 85 );
			AddSkill( index, SkillName.Tailoring, 65.0, 70.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 25 );

			// Cooking
			index = AddCraft( typeof( StoneOvenEastDeed ), "Cooking", "Stone Oven (East)", 68.4, 80.0, typeof( Log ), "Log", 85 );
			AddSkill( index, SkillName.Tinkering, 50.0, 55.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 125 );
			index = AddCraft( typeof( StoneOvenSouthDeed ), "Cooking", "Stone Oven (South)", 68.4, 80.0, typeof( Log ), "Log", 85 );
			AddSkill( index, SkillName.Tinkering, 50.0, 55.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 125 );
			index = AddCraft( typeof( FlourMillEastDeed ), "Cooking", "Flour Mill (East)", 94.7, 115.0, typeof( Log ), "Log", 100 );
			AddSkill( index, SkillName.Tinkering, 50.0, 55.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 50 );
			index = AddCraft( typeof( FlourMillSouthDeed ), "Cooking", "Flour Mill (South)", 94.7, 115.0, typeof( Log ), "Log", 100 );
			AddSkill( index, SkillName.Tinkering, 50.0, 55.0 );
			AddRes( index, typeof( IronIngot ), "Iron Ingots", 50 );
			AddCraft( typeof( WaterTroughEastDeed ), "Cooking", "Water Trough (East)", 94.7, 115.0, typeof( Log ), "Log", 150 );
			AddCraft( typeof( WaterTroughSouthDeed ), "Cooking", "Water Trough (South)", 94.7, 115.0, typeof( Log ), "Log", 150 );
		}
	}
}