using System;
using Server.Network;

namespace Server.Menus.Questions
{
	public class StuckMenu : QuestionMenu
	{
		private static Point3D[] m_Locations = new Point3D[]
			{
				new Point3D( 1520, 1617, 10 ),
				new Point3D( 1916, 2825,  4 ),
				new Point3D( 2906,  799,  0 ),
				new Point3D( 2527,  479, 15 ),
				new Point3D(  496, 1090,  0 ),
				new Point3D( 2235, 1220,  0 )
			};

		private Mobile m_Mobile, m_Sender;
		private bool m_MarkUse;

		public StuckMenu( Mobile beholder, Mobile beheld, bool markUse ) : base( String.Format( "Choose {0} destination", beholder == beheld ? "your" : "their" ), new string[]{ "Britain", "Trinsic", "Vesper", "Minoc", "Yew", "Cove" } )
		{
			m_Sender = beholder;
			m_Mobile = beheld;
			m_MarkUse = markUse;
		}

		public override void OnCancel( NetState state )
		{
			if ( m_Mobile == m_Sender )
				m_Mobile.SendLocalizedMessage( 1010588 ); // You choose not to go to any city.
		}

		public override void OnResponse( NetState state, int index )
		{
			if ( m_MarkUse )
			{
				m_Mobile.SendLocalizedMessage( 1010589 ); // You will be teleported within the next two minutes.

				new TeleportTimer( m_Mobile, m_Locations[index], TimeSpan.FromMinutes( 2.0 ) ).Start();

				m_Mobile.UsedStuckMenu();
			}
			else
			{
				new TeleportTimer( m_Mobile, m_Locations[index], TimeSpan.Zero ).Start();
			}
		}

		private class TeleportTimer : Timer
		{
			private Mobile m_Mobile;
			private Point3D m_Location;
			private DateTime m_End;
			private DateTime m_NextMessage;

			public TeleportTimer( Mobile m, Point3D loc, TimeSpan delay ) : base( TimeSpan.Zero, TimeSpan.FromSeconds( 1.0 ) )
			{
				Priority = TimerPriority.TwoFiftyMS;

				m_Mobile = m;
				m_Location = loc;
				m_End = DateTime.Now + delay;
				m_NextMessage = DateTime.Now + TimeSpan.FromSeconds( 5.0 );
			}

			protected override void OnTick()
			{
				if ( DateTime.Now > m_End )
				{
					if ( m_Mobile.Map != Map.Trammel && m_Mobile.Map != Map.Felucca )
						m_Mobile.Map = Map.Felucca;

					m_Mobile.Frozen = false;
					m_Mobile.Location = m_Location;

					Stop();
				}
				else
				{
					m_Mobile.Frozen = true;

					if ( DateTime.Now > m_NextMessage )
					{
						m_NextMessage = DateTime.Now + TimeSpan.FromSeconds( 15.0 );

						m_Mobile.SendMessage( "You will be teleported in {0} seconds.", (int)(m_End - DateTime.Now).TotalSeconds );
					}
				}
			}
		}
	}
}