using System;
using System.Collections;
using Server;
using Server.Gumps;
using Server.Network;
using Server.Mobiles;

namespace Server.Gumps
{
	public class ReportMurdererGump : Gump
	{
		private int m_Idx;
		private ArrayList m_Killers;
		private Mobile m_Victum;

		public static void Initialize()
		{
			EventSink.PlayerDeath += new PlayerDeathEventHandler( EventSink_PlayerDeath );
			EventSink.Speech += new SpeechEventHandler( EventSink_Speech );
		}
 
		public static void EventSink_PlayerDeath( PlayerDeathEventArgs e )
		{
			Mobile m = e.Mobile;
			ArrayList killers = new ArrayList();
			ArrayList toGive = new ArrayList();

			foreach ( AggressorInfo ai in m.Aggressors )
			{
				if ( ai.Attacker.Player && ai.CanReportMurder && !ai.Reported )
				{
					killers.Add( ai.Attacker );
					ai.Reported = true;
				}

				if ( (DateTime.Now - ai.LastCombatTime) < TimeSpan.FromSeconds( 30.0 ) && !toGive.Contains( ai.Defender ) )
					toGive.Add( ai.Attacker );
			}

			foreach ( AggressorInfo ai in m.Aggressed )
			{
				if ( (DateTime.Now - ai.LastCombatTime) < TimeSpan.FromSeconds( 30.0 ) && !toGive.Contains( ai.Defender ) )
					toGive.Add( ai.Defender );
			}

			foreach ( Mobile g in toGive )
			{
				int n = Notoriety.Compute( g, m );

				int karmaToGive = (n == Notoriety.Innocent ? (g.Karma > -2500 ? -850 : -40) : (n == Notoriety.Criminal || n == Notoriety.Murderer) ? 40 : 0);
				int fameToGive = m.Fame / 200;

				if ( (g.Karma + karmaToGive) > 10000 )
					karmaToGive = 10000 - g.Karma;
				else if ( (g.Karma + karmaToGive) < -10000 )
					karmaToGive = -10000 - g.Karma;

				if ( (g.Fame + fameToGive) > 10000 )
					fameToGive = 10000 - g.Fame;
				else if ( (g.Fame + fameToGive) < 0 )
					fameToGive = -g.Fame;

				g.Karma += karmaToGive;
				g.Fame += fameToGive;

				if ( fameToGive >= 40 )
					g.SendLocalizedMessage( 1019054 ); // You have gained a lot of fame.
				else if ( fameToGive >= 25 )
					g.SendLocalizedMessage( 1019053 ); // You have gained a good amount of fame.
				else if ( fameToGive >= 10 )
					g.SendLocalizedMessage( 1019052 ); // You have gained some fame.
				else if ( fameToGive >= 1 )
					g.SendLocalizedMessage( 1019051 ); // You have gained a little fame.
				else if ( fameToGive <= -40 )
					g.SendLocalizedMessage( 1019058 ); // You have lost a lot of fame.
				else if ( fameToGive <= -25 )
					g.SendLocalizedMessage( 1019057 ); // You have lost a good amount of fame.
				else if ( fameToGive <= -10 )
					g.SendLocalizedMessage( 1019056 ); // You have lost some fame.
				else if ( fameToGive <= -1 )
					g.SendLocalizedMessage( 1019055 ); // You have lost a little fame.

				if ( karmaToGive >= 40 )
					g.SendLocalizedMessage( 1019062 ); // You have gained a lot of karma.
				else if ( karmaToGive >= 25 )
					g.SendLocalizedMessage( 1019060 ); // You have gained some karma.
				else if ( karmaToGive >= 10 )
					g.SendLocalizedMessage( 1019060 ); // You have gained some karma.
				else if ( karmaToGive >= 1 )
					g.SendLocalizedMessage( 1019059 ); // You have gained a little karma.
				else if ( karmaToGive <= -40 )
					g.SendLocalizedMessage( 1019066 ); // You have lost a lot of karma.
				else if ( karmaToGive <= -25 )
					g.SendLocalizedMessage( 1019065 ); // You have lost a good amount of karma.
				else if ( karmaToGive <= -10 )
					g.SendLocalizedMessage( 1019064 ); // You have lost some karma.
				else if ( karmaToGive <= -1 )
					g.SendLocalizedMessage( 1019063 ); // You have lost a little karma.
			}

			if ( killers.Count > 0 )
				m.SendGump( new ReportMurdererGump( m, killers ) );
		}

		public static void EventSink_Speech( SpeechEventArgs e )
		{
			if ( e.HasKeyword( 0x32 ) )//"*i must consider my sins*
			{
				e.Mobile.SendMessage( "Short Term Murders : {0}", e.Mobile.ShortTermMurders );
				e.Mobile.SendMessage( "Long Term Murders : {0}",  e.Mobile.Kills );
			}
		}

		public ReportMurdererGump( Mobile victum, ArrayList killers ) : this( victum, killers, 0 )
		{
		}

		private ReportMurdererGump( Mobile victum, ArrayList killers, int idx ) : base( 0, 0 )
		{
			m_Killers = killers;
			m_Victum = victum;
			m_Idx = idx;
			BuildGump();
		}

		private void BuildGump() 
		{
			AddBackground( 265, 205, 320, 290, 5054 );
			Closable = false;
			Resizable = false;
			
			AddPage( 0 );      			
			
			AddImageTiled( 225, 175, 50, 45, 0xCE );   //Top left corner
			AddImageTiled( 267, 175, 315, 44, 0xC9 );  //Top bar
			AddImageTiled( 582, 175, 43, 45, 0xCF );   //Top right corner
			AddImageTiled( 225, 219, 44, 270, 0xCA );  //Left side
			AddImageTiled( 582, 219, 44, 270, 0xCB );  //Right side
			AddImageTiled( 225, 489, 44, 43, 0xCC );   //Lower left corner
			AddImageTiled( 267, 489, 315, 43, 0xE9 );  //Lower Bar
			AddImageTiled( 582, 489, 43, 43, 0xCD );   //Lower right corner

			AddPage( 1 );
			
			AddHtml( 260, 234, 300, 140, ((Mobile)m_Killers[m_Idx]).Name, false, false ); // Player's Name
			AddHtmlLocalized( 260, 254, 300, 140, 1049066, false, false ); // Would you like to report...

			AddButton( 260, 300, 0xFA5, 0xFA7, 1, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 300, 300, 300, 50, 1046362, false, false ); // Yes
			      	
			AddButton( 360, 300, 0xFA5, 0xFA7, 2, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 400, 300, 300, 50, 1046363, false, false ); // No      
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			Mobile from = state.Mobile;

			switch ( info.ButtonID )
			{
				case 1: 
				{            
					Mobile killer = (Mobile)m_Killers[m_Idx];
					if ( killer != null && !killer.Deleted )
					{
						killer.Kills++;
						killer.ShortTermMurders++;
						killer.SendLocalizedMessage( 1049067 );//You have been reported for murder!
						if ( killer.Kills == 5 )
							killer.SendLocalizedMessage( 502134 );//You are now known as a murderer!
					}
					break; 
				}
				case 2: 
				{
					break; 
				}
			}

			m_Idx++;
			if ( m_Idx < m_Killers.Count )
				from.SendGump( new ReportMurdererGump( from, m_Killers, m_Idx ) );
		}
	}
}
