using System;
using System.Collections;
using Server.Network;

namespace Server.Items
{
	public abstract class BaseArmor : Item, IArmor
	{
		private int m_ArBaseValue;
		private int m_MaxHitPoints;
		private int m_HitPoints;
		private Mobile m_Crafter;
		private ArmorQuality m_Quality;
		private ArmorDurabilityLevel m_Durability;
		private ArmorProtectionLevel m_Protection;
		private ArmorMaterialType m_Material;
		private ArmorBodyType m_BodyPos;
		private OreInfo m_Ore;
		private int m_StrBonus;
		private int m_DexBonus;
		private int m_IntBonus;
		private int m_StrReq;
		private int m_DexReq;
		private int m_IntReq;
		private ArmorMeditationAllowance m_Meditate;
		private bool m_Identified;

		private static double[] m_ArmorScalars = { 0.07, 0.07, 0.14, 0.15, 0.22, 0.35 };

		public static double[] ArmorScalars
		{
			get
			{
				return m_ArmorScalars;
			}
			set
			{
				m_ArmorScalars = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorBodyType BodyPosition
		{
			get
			{
				return m_BodyPos;
			} 
			set
			{
				if ( m_BodyPos != value )
				{
					m_BodyPos = value;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorMaterialType MaterialType
		{
			get
			{
				return m_Material;
			}
			set
			{
				if ( m_Material != value )
				{
					m_Material = value;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Identified
		{
			get
			{
				return m_Identified;
			}
			set
			{
				m_Identified = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public OreInfo OreType
		{
			get
			{
				return m_Ore;
			}
			set
			{
				if ( m_Ore != value )
				{
					m_Ore = value;
					Hue = m_Ore.Hue;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int StrBonus
		{
			get
			{
				return m_StrBonus;
			}
			set
			{
				m_StrBonus = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int DexBonus
		{
			get
			{
				return m_DexBonus;
			}
			set
			{
				m_DexBonus = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int IntBonus
		{
			get
			{
				return m_IntBonus;
			}
			set
			{
				m_IntBonus = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int StrRequirement
		{
			get
			{
				return m_StrReq;
			}
			set
			{
				m_StrReq = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int DexRequirement
		{
			get
			{
				return m_DexReq;
			}
			set
			{
				m_DexReq = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int IntRequirement
		{
			get
			{
				return m_IntReq;
			}
			set
			{
				m_IntReq = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int BaseArmorRating
		{
			get
			{
				return m_ArBaseValue;
			}
			set
			{
				if ( m_ArBaseValue != value )
				{
					m_ArBaseValue = value;

					Invalidate();
				}
			}
		}

		public virtual double ArmorScalar
		{
			get
			{
				int pos = (int)m_BodyPos;

				if ( pos >= 0 && pos < m_ArmorScalars.Length )
					return m_ArmorScalars[pos];

				return 1.0;
			}
		}

		public override void OnAdded( object parent )
		{
			if ( parent is Mobile )
				((Mobile)parent).Delta( MobileDelta.Armor ); // Tell them armor rating has changed
		}

		public override void OnRemoved( object parent )
		{
			if ( parent is Mobile )
				((Mobile)parent).Delta( MobileDelta.Armor ); // Tell them armor rating has changed
		}

		public virtual double ArmorRating
		{
			get
			{
				int ar = m_ArBaseValue;

				ar += m_Ore.Level * 2;

				if ( m_Protection != ArmorProtectionLevel.Regular )
					ar += 10 + (5 * (int)m_Protection);

				switch ( m_Material )
				{
					case ArmorMaterialType.Spined: ar += 10; break;
					case ArmorMaterialType.Horned: ar += 13; break;
					case ArmorMaterialType.Barbed: ar += 16; break;
				}

				ar += -8 + (8 * (int)m_Quality);

				return ar * ArmorScalar;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int MaxHitPoints
		{
			get
			{
				return m_MaxHitPoints;
			}
			set 
			{
				m_MaxHitPoints = value;
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public int HitPoints
		{
			get 
			{
				return m_HitPoints;
			}
			set 
			{
				if ( value != m_HitPoints && MaxHitPoints != 0 )
				{
					m_HitPoints = value;

					if ( m_HitPoints < 0 )
						Delete();
					else if ( m_HitPoints > MaxHitPoints )
						m_HitPoints = MaxHitPoints;
				}
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Crafter
		{
			get
			{
				return m_Crafter;
			}
			set
			{
				m_Crafter = value;
			}
		}

		
		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorQuality Quality
		{
			get
			{
				return m_Quality;
			}
			set
			{
				if ( m_Quality != value )
				{
					m_Quality = value;

					Invalidate();
				}
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorDurabilityLevel Durability
		{
			get
			{
				return m_Durability;
			}
			set
			{
				m_Durability = value;
			}
		}


		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorProtectionLevel ProtectionLevel
		{
			get
			{
				return m_Protection;
			}
			set
			{
				if ( m_Protection != value )
				{
					m_Protection = value;

					Invalidate();
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public ArmorMeditationAllowance MeditationAllowance
		{
			get
			{
				return m_Meditate;
			}
			set
			{
				m_Meditate = value;
			}
		}

		protected void Invalidate()
		{
			if ( Parent is Mobile )
				((Mobile)Parent).Delta( MobileDelta.Armor ); // Tell them armor rating has changed
		}

		public BaseArmor( Serial serial ) :  base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 );//version

			writer.Write( (bool) m_Identified );

			writer.Write( (int)m_ArBaseValue );
			writer.Write( (int)m_MaxHitPoints );
			writer.Write( (int)m_HitPoints );
			writer.Write( m_Crafter );
			writer.Write( (int)m_Quality );
			writer.Write( (int)m_Durability );
			writer.Write( (int)m_Protection );
			writer.Write( (int)m_Material );
			writer.Write( (int)m_BodyPos );
			writer.Write( (int)m_Ore.Level );
			writer.Write( (int)m_StrBonus );
			writer.Write( (int)m_DexBonus );
			writer.Write( (int)m_IntBonus );
			writer.Write( (int)m_StrReq );
			writer.Write( (int)m_DexReq );
			writer.Write( (int)m_IntReq );
			writer.Write( (int)m_Meditate );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
					m_Identified = reader.ReadBool();
					goto case 0;
				case 0:
					m_ArBaseValue = reader.ReadInt();
					m_MaxHitPoints = reader.ReadInt();
					m_HitPoints = reader.ReadInt();
					m_Crafter = reader.ReadMobile();
					m_Quality = (ArmorQuality)reader.ReadInt();
					m_Durability = (ArmorDurabilityLevel)reader.ReadInt();
					m_Protection = (ArmorProtectionLevel)reader.ReadInt();
					m_Material = (ArmorMaterialType)reader.ReadInt();
					m_BodyPos = (ArmorBodyType)reader.ReadInt();
					switch ( reader.ReadInt() )
					{
						case 1: m_Ore = OreInfo.DullCopper; break;
						case 2: m_Ore = OreInfo.ShadowIron; break;
						case 3: m_Ore = OreInfo.Copper; break;
						case 4: m_Ore = OreInfo.Bronze; break;
						case 5: m_Ore = OreInfo.Gold; break;
						case 6: m_Ore = OreInfo.Agapite; break;
						case 7: m_Ore = OreInfo.Verite; break;
						case 8: m_Ore = OreInfo.Valorite; break;
						default:
						case 0: m_Ore = OreInfo.Iron; break;
					}
					m_StrBonus = reader.ReadInt();
					m_DexBonus = reader.ReadInt();
					m_IntBonus = reader.ReadInt();
					m_StrReq = reader.ReadInt();
					m_DexReq = reader.ReadInt();
					m_IntReq = reader.ReadInt();
					m_Meditate = (ArmorMeditationAllowance)reader.ReadInt();
					break;
			}
		}

		public BaseArmor( int itemID, OreInfo Ore, int BaseRating, ArmorMaterialType Material, ArmorBodyType position ) :  base( itemID )
		{
			m_ArBaseValue = BaseRating;
			m_Material = Material;
			m_BodyPos = position;
			m_Ore = Ore;

			m_Quality = ArmorQuality.Regular;
			m_Durability = ArmorDurabilityLevel.Regular;
			m_Crafter = null;

			Hue = Ore.Hue;

			this.Layer = (Layer)TileData.ItemTable[itemID].Quality;
		}

		public virtual int OnHit( BaseWeapon weapon, int damageTaken )
		{
			int ArDmg = 0;
			double HalfAr = ArmorRating / 2.0;
			int Absorbed = (int)(HalfAr + HalfAr*Utility.RandomDouble());

			if ( Absorbed > 1 )
			{
				if ( ( m_Material == ArmorMaterialType.Plate && weapon.Type == WeaponType.Bashing ) ||
					( m_Material == ArmorMaterialType.Cloth && weapon.Type == WeaponType.Slashing ) )
				{
					ArDmg = Absorbed / 10;
				} 
				else
				{
					ArDmg = Utility.Random( 2 );
					if ( Absorbed > BaseArmorRating / 2 )
						ArDmg++;//extra point for a lot of damage
				}

				HitPoints -= ArDmg;
			}

			damageTaken -= Absorbed;
			if ( damageTaken < 0 ) 
				damageTaken = 0;

			return damageTaken;
		}

		public override void OnSingleClick( Mobile from )
		{
			ArrayList attrs = new ArrayList();

			if ( m_Quality == ArmorQuality.Exceptional )
				attrs.Add( new EquipInfoAttribute( 1018305 - (int)m_Quality ) );

			if ( m_Identified )
			{
				if ( m_Durability != ArmorDurabilityLevel.Regular )
					attrs.Add( new EquipInfoAttribute( 1038000 + (int)m_Durability ) );

				if ( m_Protection > ArmorProtectionLevel.Regular && m_Protection <= ArmorProtectionLevel.Invulnerability )
					attrs.Add( new EquipInfoAttribute( 1038005 + (int)m_Protection ) );
			}
			else if ( m_Durability != ArmorDurabilityLevel.Regular || (m_Protection > ArmorProtectionLevel.Regular && m_Protection <= ArmorProtectionLevel.Invulnerability) )
			{
				attrs.Add( new EquipInfoAttribute( 1038000 ) ); // Unidentified
			}

			EquipmentInfo eqInfo = new EquipmentInfo( 1020000 + ItemID, m_Crafter, false, (EquipInfoAttribute[])attrs.ToArray( typeof( EquipInfoAttribute ) ) );

			from.Send( new DisplayEquipmentInfo( this, eqInfo ) );
		}
	}
}
