// LockPick by SickLab

using System;
using Server.Network;
using Server.Targeting;
using Server.Items;

namespace Server.Items
{
	public interface ILockpickable : IPoint2D
	{
		int LockLevel
		{
			get;
			set;
		}

		bool Locked
		{
			get;
			set;
		}

		void LockPick( Mobile from );
	}



	[FlipableAttribute( 0x14fc, 0x14fb )]
	public class Lockpick : Item
	{
		[Constructable]
		public Lockpick() : this( 1 )
		{
		}

		[Constructable]
		public Lockpick( int amount ) : base( 0x14FC )
		{
			Stackable = true;
			Amount = amount;
			Weight = 0.1;
		}

		public Lockpick( Serial serial ) : base( serial )
		{
		}

		public override Item Dupe( int amount )
		{
			return base.Dupe( new Lockpick(), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnDoubleClick( Mobile from )
		{
			from.SendLocalizedMessage( 502068 ); // What do you want to pick?
			from.Target = new InternalTarget( this );
		}

		private class InternalTarget : Target
		{
			private Lockpick m_Item;

			public InternalTarget( Lockpick item ) : base( 1, false, TargetFlags.None )
			{
				m_Item = item;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( m_Item.Deleted ) return;


				if ( targeted is ILockpickable )
				{
					Item item = (Item)targeted;
					from.Direction = from.GetDirectionTo( item );

					if ( ((ILockpickable)targeted).Locked )
					{
						from.PlaySound( 0x241 );

						new InternalTimer( from, (ILockpickable)targeted, m_Item ).Start();
					}
					else
					{
						// The door is not locked
						from.SendLocalizedMessage( 502069 ); // This does not appear to be locked
					}
				}
				else
				{
					from.SendLocalizedMessage( 501666 ); // You can't unlock that!

				}
			}

			private class InternalTimer : Timer
			{
				private Mobile m_From;
				private ILockpickable m_Item;
				private Lockpick m_Lockpick;
			
				public InternalTimer( Mobile from, ILockpickable item, Lockpick lockpick ) : base( TimeSpan.FromSeconds( 3.0 ) )
				{
					m_From = from;
					m_Item = item;
					m_Lockpick = lockpick;
				}

				protected void BrokeLockPickTest()
				{
					// When failed, a 25% chance to broke the lockpick
					if ( Utility.Random( 4 ) == 0 )
					{
						m_From.SendLocalizedMessage( 502074 ); // You broke the lockpick

						m_Lockpick.Consume();
					}
				}
				
				protected override void OnTick()
				{
					if ( m_From.GetDistanceToSqrt( m_Item ) > 3 )
					{
						// The player goes away... do nothing
						return;
					}

					if ( m_Item.LockLevel == 0)
					{
						// LockLevel of 0 means that the door can't be picklocked
						m_From.SendLocalizedMessage( 502073 ); // This lock cannot be picked by normal means
						BrokeLockPickTest();
						return;
					}

					if ( m_From.Skills[SkillName.Lockpicking].Value < m_Item.LockLevel )
					{
						// Do some training to gain skills
						m_From.CheckSkill( SkillName.Lockpicking, 0, m_Item.LockLevel );
						
						// The LockLevel is higher thant the LockPicking of the player
						m_From.SendLocalizedMessage( 502072 ); // You don't see how that lock can be manipulated
						BrokeLockPickTest();
						return;
					}

					if ( m_From.CheckSkill( SkillName.Lockpicking, m_Item.LockLevel, 100 ) )
					{
						// Succeded!  Pick the lock!
						m_From.SendLocalizedMessage( 502076 ); // The lock quickly yields to your skill
						m_From.PlaySound( 0x4A );
						m_Item.LockPick( m_From );
					}
					else
					{
						// The Player failed to pick the lock
						m_From.SendLocalizedMessage( 502075 ); // You are unable to pick the lock
						BrokeLockPickTest();
					}
				}
			}
		}
	}
}