using System;
using System.Collections;
using Server;
using Server.Items;
using Server.Multis.Deeds;
using Server.Regions;
using Server.Targeting;
using Server.Accounting;

namespace Server.Multis
{
	public abstract class BaseHouse : BaseMulti
	{
		private const int MaxCoOwners = 15;

		private HouseRegion m_Region;
		private HouseSign m_Sign;
		private TrashBarrel m_Trash;
		private ArrayList m_Doors;

		private Mobile m_Owner;
		private ArrayList m_Bans;
		private ArrayList m_CoOwners;
		private ArrayList m_Friends;

		private ArrayList m_LockDowns;
		private ArrayList m_Secures;

		private int m_MaxLockDowns;
		private int m_MaxSecures;

		private static Hashtable m_Table = new Hashtable();

		public BaseHouse( int multiID, Mobile owner, Rectangle2D[] rects, int MaxLockDown, int MaxSecure ) : base( multiID | 0x4000 )
		{
			m_Doors = new ArrayList();
			m_LockDowns = new ArrayList();
			m_Secures = new ArrayList();

			m_CoOwners = new ArrayList();
			m_Friends = new ArrayList();
			m_Bans = new ArrayList();

			m_Region = new HouseRegion( this );

			m_Owner = owner;

			m_MaxLockDowns = MaxLockDown;
			m_MaxSecures = MaxSecure;

			m_Region.Coords = new ArrayList( rects );

			ArrayList list = (ArrayList)m_Table[owner];

			if ( list == null )
				m_Table[owner] = list = new ArrayList();

			list.Add( this );
		}

		public BaseHouse( Serial serial ) : base( serial )
		{
		}

		public override void OnMapChange()
		{
			if ( m_LockDowns == null )
				return;

			m_Region.Map = this.Map;

			if ( m_Sign != null )
				m_Sign.Map = this.Map;

			if ( m_Trash != null )
				m_Trash.Map = this.Map;

			if ( m_Doors != null )
			{
				foreach ( Item item in m_Doors )
				{
					item.Map = this.Map;
				}
			}

			if ( m_LockDowns != null )
			{
				foreach ( Item item in m_LockDowns )
				{
					item.Map = this.Map;
				}
			}
		}

		public virtual void ChangeSignType( int itemID )
		{
			if ( m_Sign != null )
				m_Sign.ItemID = itemID;
		}

		public override void OnLocationChange( Point3D oldLocation )
		{
			if ( m_LockDowns == null )
				return;

			int x = base.Location.X - oldLocation.X;
			int y = base.Location.Y - oldLocation.Y;
			int z = base.Location.Z - oldLocation.Z;

			if ( m_Sign != null )
				m_Sign.Location = new Point3D( m_Sign.X + x, m_Sign.Y + y, m_Sign.Z + z );

			if ( m_Trash != null )
				m_Trash.Location = new Point3D( m_Trash.X + x, m_Trash.Y + y, m_Trash.Z + z );

			ArrayList newCoords = new ArrayList( m_Region.Coords.Count );
			foreach ( Rectangle2D rect in m_Region.Coords )
			{
				newCoords.Add( new Rectangle2D( this.X+rect.Start.X, this.Y+rect.Start.Y, rect.Width, rect.Height ) );
			}
			m_Region.Coords = newCoords;
			m_Region.GoLocation = new Point3D( m_Region.GoLocation.X + x, m_Region.GoLocation.Y + y, m_Region.GoLocation.Z + z );

			if ( m_Doors != null )
			{
				foreach ( Item item in m_Doors )
				{
					if ( !item.Deleted )
						item.Location = new Point3D( item.X + x, item.Y + y, item.Z + z );
				}
			}

			if ( m_LockDowns != null )
			{
				foreach ( Item item in m_LockDowns )
				{
					if ( !item.Deleted )
						item.Location = new Point3D( item.X + x, item.Y + y, item.Z + z );
				}
			}
		}

		public BaseDoor AddEastDoor( int x, int y, int z )
		{
			return AddEastDoor( true, x, y, z );
		}

		public BaseDoor AddEastDoor( bool wood, int x, int y, int z )
		{
			BaseDoor door = MakeDoor( wood, DoorFacing.SouthCW );

			AddDoor( door, x, y, z );

			return door;
		}

		public BaseDoor AddSouthDoor( int x, int y, int z )
		{
			return AddSouthDoor( true, x, y, z );
		}

		public BaseDoor AddSouthDoor( bool wood, int x, int y, int z )
		{
			BaseDoor door = MakeDoor( wood, DoorFacing.WestCW );

			AddDoor( door, x, y, z );

			return door;
		}

		public BaseDoor AddEastDoor( int x, int y, int z, uint k )
		{
			return AddEastDoor( true, x, y, z, k );
		}

		public BaseDoor AddEastDoor( bool wood, int x, int y, int z, uint k )
		{
			BaseDoor door = MakeDoor( wood, DoorFacing.SouthCW );

			door.Locked = true;
			door.KeyValue = k;

			AddDoor( door, x, y, z );

			return door;
		}

		public BaseDoor AddSouthDoor( int x, int y, int z, uint k )
		{
			return AddSouthDoor( true, x, y, z, k );
		}

		public BaseDoor AddSouthDoor( bool wood, int x, int y, int z, uint k )
		{
			BaseDoor door = MakeDoor( wood, DoorFacing.WestCW );

			door.Locked = true;
			door.KeyValue = k;

			AddDoor( door, x, y, z );

			return door;
		}

		public BaseDoor[] AddSouthDoors( int x, int y, int z, uint k )
		{
			return AddSouthDoors( true, x, y, z, k );
		}

		public BaseDoor[] AddSouthDoors( bool wood, int x, int y, int z, uint k )
		{
			BaseDoor westDoor = MakeDoor( wood, DoorFacing.WestCW );
			BaseDoor eastDoor = MakeDoor( wood, DoorFacing.EastCCW );

			westDoor.Locked = true;
			eastDoor.Locked = true;

			westDoor.KeyValue = k;
			eastDoor.KeyValue = k;

			westDoor.Link = eastDoor;
			eastDoor.Link = westDoor;

			AddDoor( westDoor, x, y, z );
			AddDoor( eastDoor, x + 1, y, z );

			return new BaseDoor[2]{ westDoor, eastDoor };
		}

		public uint CreateKeys( Mobile m )
		{
			uint value = Key.RandomValue();

			Key packKey = new Key( KeyType.Gold );
			Key bankKey = new Key( KeyType.Gold );

			packKey.KeyValue = value;
			bankKey.KeyValue = value;

			packKey.LootType = LootType.Newbied;
			bankKey.LootType = LootType.Newbied;

			BankBox box = m.BankBox;

			if ( box == null || !box.TryDropItem( m, bankKey, false ) )
				bankKey.Delete();

			m.AddToBackpack( packKey );

			return value;
		}

		public BaseDoor[] AddSouthDoors( int x, int y, int z )
		{
			return AddSouthDoors( true, x, y, z, false );
		}

		public BaseDoor[] AddSouthDoors( bool wood, int x, int y, int z, bool inv )
		{
			BaseDoor westDoor = MakeDoor( wood, inv ? DoorFacing.WestCCW : DoorFacing.WestCW );
			BaseDoor eastDoor = MakeDoor( wood, inv ? DoorFacing.EastCW : DoorFacing.EastCCW );

			westDoor.Link = eastDoor;
			eastDoor.Link = westDoor;

			AddDoor( westDoor, x, y, z );
			AddDoor( eastDoor, x + 1, y, z );

			return new BaseDoor[2]{ westDoor, eastDoor };
		}

		public BaseDoor MakeDoor( bool wood, DoorFacing facing )
		{
			if ( wood )
				return new DarkWoodHouseDoor( facing );
			else
				return new MetalHouseDoor( facing );
		}

		public void AddDoor( BaseDoor door, int xoff, int yoff, int zoff )
		{
			door.MoveToWorld( new Point3D( xoff+this.X, yoff+this.Y, zoff+this.Z ), this.Map );
			m_Doors.Add( door );
		}

		public void AddTrashBarrel( Mobile from )
		{
			if ( m_Trash == null || m_Trash.Deleted )
			{
				m_Trash = new TrashBarrel();

				m_Trash.Movable = false;
				m_Trash.MoveToWorld( from.Location, from.Map );

				from.SendLocalizedMessage( 502121 ); /* You have a new trash barrel.
													  * Three minutes after you put something in the barrel, the trash will be emptied.
													  * Be forewarned, this is permanent! */
			}
			else
			{
				m_Trash.MoveToWorld( from.Location, from.Map );
			}
		}

		public void SetSign( int xoff, int yoff, int zoff )
		{
			m_Sign = new HouseSign( this );
			m_Sign.MoveToWorld( new Point3D( this.X + xoff, this.Y + yoff, this.Z + zoff ), this.Map );
		}

		private void SetLockdown( Item i, bool locked )
		{
			if ( m_LockDowns == null )
				return;

			i.Movable = !locked;
			if ( locked )
				m_LockDowns.Add( i );
			else
				m_LockDowns.Remove( i );

			foreach ( Item c in i.Items )
				SetLockdown( c, locked );
		}

		public void LockDown( Mobile m, Item item )
		{
			if ( !IsCoOwner( m ) )
				return;

			if ( item.Movable && !IsSecure( item ) )
			{
				if ( item.RootParent != null )
				{
					if ( item.RootParent is Mobile )
					{
						m.SendLocalizedMessage( 1005525 );//That is not in your house
						return;
					}
					else if ( Region.Find( ((Item)item.RootParent).Location, m.Map ) != m_Region )
					{
						m.SendLocalizedMessage( 1005525 );//That is not in your house
						return;
					}
				}
				else if ( Region.Find( item.Location, item.Map ) != m_Region )
				{
					m.SendLocalizedMessage( 1005525 );//That is not in your house
				}
				else 
				{
					if ( this.LockDownCount < m_MaxLockDowns )
					{
						//TidyItemList( m_LockDowns );
						SetLockdown( item, true );
					}
					else
					{
						m.SendLocalizedMessage( 1005379 );//That would exceed the maximum lock down limit for this house
					}
				}
			} 
			else if ( m_LockDowns.IndexOf( item ) != -1 )
			{
				m.SendLocalizedMessage( 1005526 );//That is already locked down
			} 
			else
			{
				m.SendLocalizedMessage( 1005377 );//You cannot lock that down
			}
		}

		public void Release( Mobile m, Item item )
		{
			if ( !IsCoOwner( m ) )
				return;

			if ( m_LockDowns.Contains( item ) )
			{
				item.PublicOverheadMessage( Server.Network.MessageType.Label, 0x3B2, 501657 );//[no longer locked down]
				SetLockdown( item, false );
				//TidyItemList( m_LockDowns );
			}
			else if ( m_Secures.Contains( item ) )
			{
				m.SendMessage( "You must unsecure that." );
			}
			else
			{
				m.SendLocalizedMessage( 501722 );//That isn't locked down...
			}
		}
		
		public void AddSecure( Mobile m, Item item )
		{
			if ( !IsCoOwner( m ) )
				return;

			if ( item.Movable && item.Parent == null && item is Container )
			{
				if ( Region.Find( item.Location, item.Map ) != m_Region )
				{
					m.SendLocalizedMessage( 1005525 );//That is not in your house
				}
				else 
				{
					if ( m_Secures.Count < m_MaxSecures )
					{
						if ( LockDownCount + 125 < m_MaxLockDowns )
						{
							m_Secures.Add( item );
							//TidyItemList( m_Secures );
							m_LockDowns.Remove( item );
							item.Movable = false;
						}
						else
						{
							m.SendLocalizedMessage( 1005379 );//That would exceed the maximum lock down limit for this house
						}
					}
					else
					{
						m.SendLocalizedMessage( 1008142 );//The maximum number of secure ites has been reached :
						m.SendMessage( MaxSecures.ToString() );
					}
				}
			} 
			else if ( m_Secures.IndexOf( item ) != -1 )
			{
				m.SendLocalizedMessage( 1005372 );//That is already secure!
			} 
			else
			{
				m.SendLocalizedMessage( 1010424 );//You cannot secure this
			}
		}

		public void ReleaseSecure( Mobile m, Item item )
		{
			if ( !IsCoOwner( m ) ) return;

			if ( item is StrongBox ) return;//can't release strong boxes

			if ( m_Secures.IndexOf( item ) != -1 )
			{
				item.Movable = true;
				item.PublicOverheadMessage( Server.Network.MessageType.Label, 0x3B2, 501656 );//[no longer secure]
				m_Secures.Remove( item );
				//TidyItemList( m_Secures );
			} 
			else 
			{
				m.SendLocalizedMessage( 501717 );//This isn't secure...
			}
		}

		public override bool Decays
		{
			get
			{
				return false;
			}
		}

		public void AddStrongBox( Mobile from )
		{
			if ( !IsCoOwner( from ) )
				return;

			if ( from == Owner )
			{
				from.SendLocalizedMessage( 502109 ); // Owners don't get a strong box
				return;
			}

			if ( m_Secures.Count+1 > m_MaxSecures )
			{
				from.SendLocalizedMessage( 1008142 );//The maximum number of secure ites has been reached :
				from.SendMessage( MaxSecures.ToString() );
				return;
			} 
			else if ( LockDownCount + 25 > m_MaxLockDowns )
			{
				from.SendLocalizedMessage( 1005379 );//That would exceed the maximum lock down limit for this house
				return;
			}


			foreach ( Container c in m_Secures )
			{
				if ( c is StrongBox )
				{
					if ( ((StrongBox)c).Owner == from )
					{
						from.SendLocalizedMessage( 502112 );//You already have a strong box
						return;
					}
				}
			}

			StrongBox sb = new StrongBox( from, this );
			m_Secures.Add( sb );
			sb.MoveToWorld( from.Location, from.Map );
		}

		public void Kick( Mobile from, Mobile targ )
		{
			if ( !IsFriend( from ) )
				return;

			if ( IsFriend( targ ) )
			{
				from.SendLocalizedMessage( 501348 ); // You cannot eject a friend of the house!
			}
			else if ( targ.AccessLevel > AccessLevel.Player && from.AccessLevel <= targ.AccessLevel )
			{
				from.SendLocalizedMessage( 501354 ); // Uh oh...a bigger boot may be required.
			}
			else if ( targ.Region != m_Region )
			{
				from.SendLocalizedMessage( 501352 ); // You may not eject someone who is not in your house!
			}
			else
			{
				targ.Location = BanLocation;
				targ.Map = Map;

				from.SendLocalizedMessage( 1042840, targ.Name ); // ~1_PLAYER NAME~ has been ejected from this house.
				targ.SendLocalizedMessage( 501341 ); /* You have been ejected from this house.
													  * If you persist in entering, you may be banned from the house.
													  */
			}
		}

		public void RemoveBan( Mobile from, Mobile targ )
		{
			if ( !IsCoOwner( from ) )
				return;

			if ( m_Bans.Contains( targ ) )
			{
				m_Bans.Remove( targ );

				from.SendLocalizedMessage( 501297 ); // The ban is lifted.
			}
		}

		public void Ban( Mobile from, Mobile targ )
		{
			if ( !IsCoOwner( from ) )
				return;

			if ( IsFriend( targ ) )
			{
				from.SendLocalizedMessage( 501348 ); // You cannot eject a friend of the house!
			}
			else if ( targ.AccessLevel > AccessLevel.Player && from.AccessLevel <= targ.AccessLevel )
			{
				from.SendLocalizedMessage( 501354 ); // Uh oh...a bigger boot may be required.
			}
			else if ( targ.Region != m_Region )
			{
				from.SendLocalizedMessage( 501352 ); // You may not eject someone who is not in your house!
			}
			else if ( m_Bans.Contains( targ ) )
			{
				from.SendLocalizedMessage( 501356 ); // This person is already banned!
			}
			else
			{
				targ.Location = BanLocation;
				targ.Map = Map;

				from.SendLocalizedMessage( 1042839, targ.Name ); // ~1_PLAYER_NAME~ has been banned from this house.
				targ.SendLocalizedMessage( 501340 ); // You have been banned from this house.

				if ( m_Bans != null )
					m_Bans.Add( targ );
			}
		}

		public void AddCoOwner( Mobile from, Mobile targ )
		{
			if ( !IsOwner( from ) )
				return;

			if ( IsFriend( targ ) )
				RemoveFriend( from, targ );
			else if ( IsBanned( targ ) )
				RemoveBan( from, targ );

			if ( IsCoOwner( targ ) )
			{
				from.SendLocalizedMessage( 501363 );//already coowner
			}
			else if ( m_CoOwners.Count > MaxCoOwners )
			{
				from.SendLocalizedMessage( 501368 );//CoOwner list full
			}
			else
			{
				m_CoOwners.Add( targ );
				targ.Delta( MobileDelta.Noto );
				//TidyMobileList( m_CoOwners );
				targ.SendLocalizedMessage( 501343 );//you have been made coowner
				from.SendMessage( "They are now a co-owner of the house." );
			}
		}

		public void RemoveCoOwner( Mobile from, Mobile targ )
		{
			if ( !IsOwner( from ) )
				return;

			if ( m_CoOwners.IndexOf( targ ) != -1 )
			{
				m_CoOwners.Remove( targ );
				//TidyMobileList( m_CoOwners );
				targ.SendLocalizedMessage( 501300 );//You have been removed as coowner
				from.SendLocalizedMessage( 501299 );//coowner removed
				foreach ( Container c in m_Secures )
				{
					if ( c is StrongBox )
					{
						if ( ((StrongBox)c).Owner == targ )
						{
							m_Secures.Remove( c );
							c.Movable = true;
							((StrongBox)c).Owner = null;
							break;
						}
					}
				}
			}
			else
			{
				from.SendMessage( "They are not a co-owner of the house." );
			}
		}

		public void AddFriend( Mobile from, Mobile targ )
		{
			if ( !IsCoOwner( from ) )
				return;

			if ( m_Friends == null )
				return;

			if ( IsBanned( targ ) )
				RemoveBan( from, targ );

			if ( IsCoOwner( targ ) )
			{
				from.SendLocalizedMessage( 501363 );//already a coowner
			}
			else if ( !m_Friends.Contains( targ ) )
			{
				m_Friends.Add( targ );
				targ.Delta( MobileDelta.Noto );
				//TidyMobileList( m_Friends );
				targ.SendLocalizedMessage( 501337 );//you have been made a friend
				from.SendMessage( "They have been made a friend of the household" );
			}
			else
			{
				from.SendLocalizedMessage( 501376 );//Already a friend
			}
		}

		public void RemoveFriend( Mobile from, Mobile targ )
		{
			if ( !IsCoOwner( from ) )
				return;

			if ( m_Friends == null )
				return;

			if ( m_Friends.Contains( targ ) )
			{
				m_Friends.Remove( targ );
				//TidyMobileList( m_Friends );
				from.SendLocalizedMessage( 501298 );
				targ.SendMessage( "You have been removed as a friend of the house." );
			}
			else
			{
				from.SendMessage( "That person is not a friend of the house." );
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int)1 ); // version

			writer.Write( BanLocation );

			writer.Write( m_Owner );

			writer.Write( (int)m_Region.Coords.Count );
			foreach( Rectangle2D rect in m_Region.Coords )
			{
				writer.Write( rect );
			}

			writer.WriteMobileList( m_CoOwners, true );
			writer.WriteMobileList( m_Friends, true );
			writer.WriteMobileList( m_Bans, true );

			writer.Write( m_Sign );
			writer.Write( m_Trash );

			writer.WriteItemList( m_Doors, true );
			writer.WriteItemList( m_LockDowns, true );
			writer.WriteItemList( m_Secures, true );

			writer.Write( (int) m_MaxLockDowns );
			writer.Write( (int) m_MaxSecures );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
			int count;

			m_Region = new HouseRegion( this );

			switch ( version )
			{
				case 1:
				{
					m_Region.GoLocation = reader.ReadPoint3D();

					goto case 0;
				}
				case 0:
				{
					m_Owner = reader.ReadMobile();

					count = reader.ReadInt();
					m_Region.Coords = new ArrayList( count );
					for(int i=0;i<count;i++)
					{
						m_Region.Coords.Add( reader.ReadRect2D() );
					}
					Region.AddRegion( m_Region );

					m_CoOwners = reader.ReadMobileList();
					m_Friends = reader.ReadMobileList();
					m_Bans = reader.ReadMobileList();

					m_Sign = reader.ReadItem() as HouseSign;
					m_Trash = reader.ReadItem() as TrashBarrel;

					m_Doors = reader.ReadItemList();
					m_LockDowns = reader.ReadItemList();
					m_Secures = reader.ReadItemList();

					m_MaxLockDowns = reader.ReadInt();
					m_MaxSecures = reader.ReadInt();

					if ( m_Owner != null )
					{
						ArrayList list = (ArrayList)m_Table[m_Owner];

						if ( list == null )
							m_Table[m_Owner] = list = new ArrayList();

						list.Add( this );
					}

					break;
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Owner
		{
			get
			{
				return m_Owner;
			}
			set
			{
				if ( m_Owner != null )
				{
					ArrayList list = (ArrayList)m_Table[m_Owner];

					if ( list == null )
						m_Table[m_Owner] = list = new ArrayList();

					list.Remove( this );
					m_Owner.Delta( MobileDelta.Noto );
				}

				m_Owner = value;

				if ( m_Owner != null )
				{
					ArrayList list = (ArrayList)m_Table[m_Owner];

					if ( list == null )
						m_Table[m_Owner] = list = new ArrayList();

					list.Add( this );
					m_Owner.Delta( MobileDelta.Noto );
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int MaxSecures
		{
			get
			{
				return m_MaxSecures;
			}
			set
			{
				m_MaxSecures = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D BanLocation
		{
			get
			{
				return m_Region.GoLocation;
			}
			set
			{
				m_Region.GoLocation = new Point3D( m_Region.GoLocation.X + value.X, m_Region.GoLocation.Y + value.Y, m_Region.GoLocation.Z + value.Z );
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int MaxLockDowns
		{
			get
			{
				return m_MaxLockDowns;
			}
			set
			{
				m_MaxLockDowns = value;
			}
		}

		public Region Region
		{
			get
			{
				return m_Region;
			}
		}

		public ArrayList CoOwners
		{
			get
			{
				return m_CoOwners;
			}
		}

		public ArrayList Friends
		{
			get
			{
				return m_Friends;
			}
		}

		public ArrayList Bans
		{
			get
			{
				return m_Bans;
			}
		}

		public ArrayList Doors
		{
			get
			{
				return m_Doors;
			}
		}

		public int LockDownCount
		{
			get
			{
				int count = 0;

				if ( m_LockDowns != null )
					count += m_LockDowns.Count;

				if ( m_Secures != null )
				{
					for ( int i = 0; i < m_Secures.Count; ++i )
					{
						Item c = (Item)m_Secures[i];

						if ( c is StrongBox )
							count += 25;
						else
							count += 125;
					}
				}

				return count;
			}
		}

		public ArrayList LockDowns
		{
			get
			{
				return m_LockDowns;
			}
		}

		public ArrayList Secures
		{
			get
			{
				return m_Secures;
			}
		}

		public HouseSign Sign
		{
			get
			{
				return m_Sign;
			}
			set
			{
				m_Sign = value;
			}
		}

		public override void OnAfterDelete()
		{
			base.OnAfterDelete();

			if ( m_Owner != null )
			{
				ArrayList list = (ArrayList)m_Table[m_Owner];

				if ( list == null )
					m_Table[m_Owner] = list = new ArrayList();

				list.Remove( this );
			}

			Region.RemoveRegion( m_Region );

			if ( m_Sign != null )
				m_Sign.Delete();

			if ( m_Doors != null )
			{
				for ( int i = 0; i < m_Doors.Count; ++i )
				{
					Item item = (Item)m_Doors[i];

					if ( item != null )
						item.Delete();
				}
			}

			if ( m_LockDowns != null )
			{
				for ( int i = 0; i < m_LockDowns.Count; ++i )
				{
					Item item = (Item)m_LockDowns[i];

					if ( item != null )
						item.Movable = true;
				}
			}
		}

		public static bool HasHouse( Mobile m )
		{
			if ( m == null )
				return false;

			ArrayList list = (ArrayList)m_Table[m];

			if ( list == null )
				return false;

			for ( int i = 0; i < list.Count; ++i )
			{
				BaseHouse h = (BaseHouse)list[i];

				if ( !h.Deleted )
					return true;
			}

			return false;
		}

		public static bool HasAccountHouse( Mobile m )
		{
			Account a = m.Account;

			if ( a == null )
				return false;

			for ( int i = 0; i < 5; ++i )
				if ( a[i] != null && HasHouse( a[i] ) )
					return true;

			return false;
		}

		public bool IsOwner( Mobile m )
		{
			if ( m == null )
				return false;

			return ( m == m_Owner || m.AccessLevel >= AccessLevel.GameMaster );
		}

		public bool IsCoOwner( Mobile m )
		{
			if ( m == null || m_CoOwners == null )
				return false;

			if ( IsOwner( m ) || m_CoOwners.Contains( m ) )
				return true;

			if ( m_Owner != null )
			{
				Account a = m_Owner.Account;

				if ( a != null )
				{
					for ( int i = 0; i < 5; ++i )
					{
						if ( a[i] == m )
							return true;
					}
				}
			}

			return false;
		}

		public void ChangeLocks( Mobile m )
		{
			uint keyValue = CreateKeys( m );

			if ( m_Doors != null )
			{
				for ( int i = 0; i < m_Doors.Count; ++i )
				{
					BaseDoor door = m_Doors[i] as BaseDoor;

					if ( door != null )
						door.KeyValue = keyValue;
				}
			}
		}

		public abstract HouseDeed GetDeed();

		public bool IsFriend( Mobile m )
		{
			if ( m == null || m_Friends == null )
				return false;

			return ( IsCoOwner( m ) || m_Friends.Contains( m ) );
		}

		public bool IsBanned( Mobile m )
		{
			if ( m == null || m == Owner || m.AccessLevel > AccessLevel.Player || m_Bans == null )
				return false;

			for ( int i = 0; i < m_Bans.Count; ++i )
			{
				Mobile c = (Mobile)m_Bans[i];

				if ( c == m )
					return true;

				Account a = c.Account;

				if ( a == null )
					continue;

				for ( int j = 0; j < 5; ++j )
				{
					if ( a[i] == m )
						return true;
				}
			}

			return false;
		}

		public bool IsLockedDown( Item i )
		{
			if ( i == null )
				return false;

			if ( m_LockDowns == null )
				return false;

			return m_LockDowns.Contains( i );
		}

		public bool IsSecure( Item i )
		{
			if ( i == null )
				return false;

			if ( m_Secures == null )
				return false;

			return m_Secures.Contains( i );
		}
	}

	public class LockdownTarget : Target
	{
		private bool m_Release;
		private BaseHouse m_House;

		public LockdownTarget( bool release, BaseHouse house ) : base( 12, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_Release = release;
			m_House = house;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsCoOwner( from ) )
				return;

			if ( targeted is Item )
			{
				if ( m_Release )
					m_House.Release( from, (Item)targeted );
				else
					m_House.LockDown( from, (Item)targeted );
			} 
			else 
			{
				from.SendLocalizedMessage( 1005377 );//You cannot lock that down
			}
		}
	}

	public class SecureTarget : Target
	{
		private bool m_Release;
		private BaseHouse m_House;

		public SecureTarget( bool release, BaseHouse house ) : base( 12, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_Release = release;
			m_House = house;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsCoOwner( from ) )
				return;

			if ( targeted is Item )
			{
				if ( m_Release )
					m_House.ReleaseSecure( from, (Item)targeted );
				else
					m_House.AddSecure( from, (Item)targeted );
			} 
			else 
			{
				from.SendLocalizedMessage( 1010424 );//You cannot secure this
			}
		}
	}

	public class HouseKickTarget : Target
	{
		private BaseHouse m_House;

		public HouseKickTarget( BaseHouse house ) : base( -1, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_House = house;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsFriend( from ) )
				return;

			if ( targeted is Mobile )
			{
				m_House.Kick( from, (Mobile)targeted );
			} 
			else 
			{
				from.SendLocalizedMessage( 501347 );//You cannot eject that from the house!
			}
		}
	}

	public class HouseBanTarget : Target
	{
		private BaseHouse m_House;
		private bool m_Banning;

		public HouseBanTarget( bool ban, BaseHouse house ) : base( -1, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_House = house;
			m_Banning = ban;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsFriend( from ) )
				return;

			if ( targeted is Mobile )
			{
				if ( m_Banning )
					m_House.Ban( from, (Mobile)targeted );
				else
					m_House.RemoveBan( from, (Mobile)targeted );
			} 
			else 
			{
				from.SendLocalizedMessage( 501347 );//You cannot eject that from the house!
			}
		}
	}

	public class CoOwnerTarget : Target
	{
		private BaseHouse m_House;
		private bool m_Add;

		public CoOwnerTarget( bool add, BaseHouse house ) : base( 12, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_House = house;
			m_Add = add;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsOwner( from ) )
				return;

			if ( targeted is Mobile )
			{
				if ( m_Add )
					m_House.AddCoOwner( from, (Mobile)targeted );
				else
					m_House.RemoveCoOwner( from, (Mobile)targeted );
			} 
			else 
			{
				from.SendLocalizedMessage( 501362 );//That can't be a coowner
			}
		}
	}

	public class HouseFriendTarget : Target
	{
		private BaseHouse m_House;
		private bool m_Add;

		public HouseFriendTarget( bool add, BaseHouse house ) : base( 12, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_House = house;
			m_Add = add;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsCoOwner( from ) )
				return;

			if ( targeted is Mobile )
			{
				if ( m_Add )
					m_House.AddFriend( from, (Mobile)targeted );
				else
					m_House.RemoveFriend( from, (Mobile)targeted );
			} 
			else 
			{
				from.SendLocalizedMessage( 501371 );//That can't be a friend
			}
		}
	}

	public class HouseOwnerTarget : Target
	{
		private BaseHouse m_House;

		public HouseOwnerTarget( BaseHouse house ) : base( 12, false, TargetFlags.None )
		{
			CheckLOS = false;

			m_House = house;
		}

		protected override void OnTarget( Mobile from, object targeted )
		{
			if ( !from.Alive || !m_House.IsOwner( from ) )
				return;

			if ( targeted is Mobile )
			{
				Mobile t = (Mobile)targeted;

				if ( t.Player )
				{
					if ( BaseHouse.HasAccountHouse( t ) )
					{
						from.SendLocalizedMessage( 501388 ); // You cannot transfer ownership to another house owner or co-owner!
					}
					else
					{
						from.SendLocalizedMessage( 501338 );//You have transfered ownership
						m_House.Owner = t;
						t.SendLocalizedMessage( 501339 );//You are now the owner.  Friends & bans removed
						m_House.Bans.Clear();
						m_House.Friends.Clear();
						m_House.CoOwners.Clear();

						m_House.ChangeLocks( t );
					}
				}
				else
				{
					from.SendLocalizedMessage( 501384 ); // Only a player can own a house!
				}
			} 
			else 
			{
				from.SendLocalizedMessage( 501384 ); // Only a player can own a house!
			}
		}
	}
}
