using System;
using System.Collections;
using Server;
using Server.Multis;

namespace Server.Items
{
	public enum PlankSide{ Port = 0, Starboard = 1 }
	public class Plank : Item, ILockable
	{
		private Boat m_Boat;
		private PlankSide m_Side;
		private bool m_Locked;
		private Timer m_CloseTimer;
		private uint m_KeyVal;

		public Plank( Boat boat, PlankSide side, uint keyval ) : base( 0x3EB1 + (int)side )
		{
			m_Boat = boat;
			m_Side = side;
			m_KeyVal = keyval;
			m_Locked = true;

			Movable = false;
		}

		public Plank( Serial serial ) : base(serial)
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 );//version

			writer.Write( m_Boat );
			writer.Write( (int)m_Side );
			writer.Write( m_Locked );
			writer.Write( m_KeyVal );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Boat = reader.ReadItem() as Boat;
					m_Side = (PlankSide)reader.ReadInt();
					m_Locked = reader.ReadBool();
					m_KeyVal = reader.ReadUInt();
					break;
				}
			}

			if ( IsOpen )
				m_CloseTimer = new CloseTimer( this );
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public PlankSide Side
		{
			get
			{
				return m_Side;
			}
			set
			{
				m_Side = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Locked
		{
			get
			{
				return m_Locked;
			}
			set
			{
				if ( m_Locked != value )
				{
					if ( value == true )
					{
						if ( Close() )
							m_Locked = true;
					}
					else
					{
						m_Locked = false;
					}
				}
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public uint KeyValue
		{
			get
			{
				return m_KeyVal;
			}
			set
			{
				m_KeyVal = value;
			}
		}

		public void Turn( Direction dir )
		{
			if ( IsOpen )
			{
				switch ( dir )
				{
					case Direction.North:
						this.ItemID = m_Side == PlankSide.Starboard ? 0x3ED4 : 0x3ED5;
						break;
					case Direction.East:
						this.ItemID = m_Side == PlankSide.Starboard ? 0x3E84 : 0x3E89;
						break;
					case Direction.South:
						this.ItemID = m_Side == PlankSide.Starboard ? 0x3ED5 : 0x3ED4;
						break;
					case Direction.West:
						this.ItemID =  m_Side == PlankSide.Starboard ? 0x3E89 : 0x3E84;
						break;
				}
			}
			else
			{
				switch ( dir )
				{
					case Direction.North:
						this.ItemID = m_Side == PlankSide.Starboard ? 0x3EB2 : 0x3EB1;
						break;
					case Direction.East:
						this.ItemID = m_Side == PlankSide.Starboard ? 0x3E85 : 0x3E8A;
						break;
					case Direction.South:
						this.ItemID = m_Side == PlankSide.Starboard ? 0x3EB1 : 0x3EB2;
						break;
					case Direction.West:
						this.ItemID =  m_Side == PlankSide.Starboard ? 0x3E8A : 0x3E85;
						break;
				}
			}
		}

		public bool IsOpen
		{
			get
			{
				return ( ItemID == 0x3ED5 || ItemID == 0x3ED4 || ItemID == 0x3E84 || ItemID == 0x3E89 );
			}
		}

		public void Open()
		{
			if ( IsOpen )
				return;

			if ( m_CloseTimer == null )
				m_CloseTimer = new CloseTimer( this );
			m_CloseTimer.Start();

			switch ( ItemID )
			{
				case 0x3EB1:
					ItemID = 0x3ED5;
					break;
				case 0x3E8A:
					ItemID = 0x3E89;
					break;
				case 0x3EB2:
					ItemID = 0x3ED4;
					break;
				case 0x3E85:
					ItemID = 0x3E84;
					break;
			}
		}

		public override bool OnMoveOver( Mobile from )
		{
			if ( IsOpen && ( from.Direction & Direction.Running ) == 0 )
			{
				int xmin=0, xmax=0, ymin=0, ymax=0;
				switch ( m_Boat.Direction )
				{
					case Direction.North:
						if ( this.Side == PlankSide.Starboard )
							xmin = this.X + m_Boat.Width/2 + 1;
						else
							xmin = this.X - m_Boat.Width/2 - 3;
						xmax = xmin+4;
						
						ymin = this.Y-2;
						ymax = ymin + 4;
						break;
					case Direction.South:
						if ( this.Side == PlankSide.Port )
							xmin = this.X + m_Boat.Width/2 + 1;
						else
							xmin = this.X - m_Boat.Width/2 - 3;
						xmax = xmin+4;
						
						ymin = this.Y-2;
						ymax = ymin + 4;
						break;
					case Direction.East:
						xmin = this.X - 2;
						xmax = xmin + 4;

						if ( this.Side == PlankSide.Starboard )
							ymin = this.Y + m_Boat.Width/2 + 1;
						else
							ymin = this.Y - m_Boat.Width/2 - 3;
						ymax = ymin+4;
						break;
					case Direction.West:
						xmin = this.X - 2;
						xmax = xmin + 4;

						if ( this.Side == PlankSide.Starboard )
							ymin = this.Y + m_Boat.Width/2 + 1;
						else
							ymin = this.Y - m_Boat.Width/2 - 3;
						ymax = ymin+4;
						break;
				}
				for ( int x=xmin;x<=xmax;x++)
				{
					for ( int y=ymin;y<=ymax;y++)
					{
						int z = Map.GetAverageZ( x, y );
						if ( ValidLand( x, y, z ) )
						{
							from.Location = new Point3D( x, y, z );
							return false;
						}
					}
				}
				return true;
			}
			else
			{
				return true;
			}
		}

		private bool ValidLand( int x, int y, int z )
		{
			if ( Map == Map.Internal )
				return false;

			bool hasSurface = false;

			Tile lt = Map.Tiles.GetLandTile( x, y );
			int lowZ = 0, avgZ = 0, topZ = 0;

			Map.GetAverageZ( x, y, ref lowZ, ref avgZ, ref topZ );
			TileFlag landFlags = TileData.LandTable[lt.ID & 0x3FFF].Flags;

			if ( (landFlags & TileFlag.Impassable) != 0 && topZ > z && (z + 16) > lowZ )
				return false;
			else if ( (landFlags & TileFlag.Impassable) == 0 && z == avgZ && !lt.Ignored )
				hasSurface = true;

			Tile[] staticTiles = Map.Tiles.GetStaticTiles( x, y, true );

			for ( int i = 0; i < staticTiles.Length; ++i )
			{
				ItemData id = TileData.ItemTable[staticTiles[i].ID & 0x3FFF];

				if ( (id.Flags & (TileFlag.Impassable | TileFlag.Surface)) != 0 && (staticTiles[i].Z + id.CalcHeight) > z && (z + 16) > staticTiles[i].Z )
				{
					return false;
				}
				else if ( (id.Flags & TileFlag.Surface) != 0 && z == (staticTiles[i].Z + id.CalcHeight) )
				{
					hasSurface = true;
				}
			}

			Sector sector = Map.GetSector( x, y );
			ArrayList items = sector.Items, mobs = sector.Mobiles;

			for ( int i = 0; i < items.Count; ++i )
			{
				Item item = (Item)items[i];

				if ( item.AtWorldPoint( x, y ) )
				{
					ItemData id = item.ItemData;

					if ( (id.Flags & (TileFlag.Impassable | TileFlag.Surface)) != 0 && (item.Z + id.CalcHeight) > z && (z + 16) > item.Z )
						return false;
					else if ( (id.Flags & TileFlag.Surface) != 0 && z == (item.Z + id.CalcHeight) )
						hasSurface = true;
				}
			}

			return hasSurface;
		}

		public bool Close()
		{
			int count = 0;
			if ( !IsOpen )
				return true;

			IPooledEnumerable eable = Map.GetMobilesInRange( Location, 0 );

			foreach( Mobile m in eable )
			{
				if ( m.Location == this.Location )
					count++;
			}

			eable.Free();

			if ( m_CloseTimer != null && m_CloseTimer.Running )
				m_CloseTimer.Stop();

			if ( count > 0 )
				return false;

			switch ( ItemID )
			{
				case 0x3ED5:
					ItemID = 0x3EB1;
					break;
				case 0x3E89:
					ItemID = 0x3E8A;
					break;
				case 0x3ED4:
					ItemID = 0x3EB2;
					break;
				case 0x3E84:
					ItemID = 0x3E85;
					break;
			}
			return true;
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( from.InRange( this.Location, 6 ) )
			{
				if ( m_Boat.Contains( from ) )
				{
					if ( IsOpen )
					{
						Close();
					}
					else if ( !Locked )
					{
						Open();
					}
					else if ( from.AccessLevel >= AccessLevel.GameMaster )
					{
						from.LocalOverheadMessage( Network.MessageType.Regular, 0x00, 502502 );//That is locked but your godly powers allow access
						Open();
					}
					else
					{	
						from.LocalOverheadMessage( Network.MessageType.Regular, 0x00, 502503 ); //That is locked.
					}
				}
				else
				{
					if ( !IsOpen )
					{
						if ( !Locked )
						{
							Open();
						}
						else if ( from.AccessLevel >= AccessLevel.GameMaster )
						{
							from.LocalOverheadMessage( Network.MessageType.Regular, 0x00, 502502 );//That is locked but your godly powers allow access
							Open();
						}
						else
						{	
							from.LocalOverheadMessage( Network.MessageType.Regular, 0x00, 502503 ); //That is locked.
						}
					}
					else
					{
						from.Location = new Point3D( this.X, this.Y, this.Z + 3 );
					}
				}
			}
		}

		private class CloseTimer : Timer
		{
			private Plank m_Plank;

			public CloseTimer( Plank plank ) : base( TimeSpan.FromSeconds( 30 ) )
			{
				m_Plank = plank;
				Priority = TimerPriority.FiveSeconds;
			}

			protected override void OnTick()
			{
				if ( m_Plank.IsOpen )
				{
					if ( !m_Plank.Close() )
					{
						Delay = TimeSpan.FromSeconds( 15.0 );
						Start();//start the timer again
					}
				}
			}
		}
	}
}
