using System;
using Server;
using Server.Mobiles;
using Server.Spells;
using Server.Spells.Seventh;
using Server.Spells.Fourth;
using Server.Spells.Sixth;

namespace Server.Regions
{
	public class TrammelDungeon : Region
	{
		public static void Initialize()
		{
			Region.AddRegion( new TrammelDungeon( "Covetous" ) );
			Region.AddRegion( new TrammelDungeon( "Deceit" ) );
			Region.AddRegion( new TrammelDungeon( "Despise" ) );
			Region.AddRegion( new TrammelDungeon( "Destard" ) );
			Region.AddRegion( new TrammelDungeon( "Hythloth" ) );
			Region.AddRegion( new TrammelDungeon( "Shame" ) );
			Region.AddRegion( new TrammelDungeon( "Wrong" ) );
		}

		public TrammelDungeon( string name ) : base( "the dungeon", name, Map.Trammel )
		{
		}

		public override bool AllowHousing( Mobile from, Point3D p )
		{
			return false;
		}

		public override bool OnBeginSpellCast( Mobile m, ISpell s )
		{
			if ( s is GateTravelSpell || s is RecallSpell || s is MarkSpell )
			{
				m.SendMessage( "You cannot cast that spell here." );
				return false;
			}
			else
			{
				return base.OnBeginSpellCast( m, s );
			}
		}

		public override double LightLevel( Mobile m, double level )
		{
			return LightCycle.DungeonLevel;
		}

		public override bool AllowHarmful( Mobile from, Mobile target )
		{
			int notor = Notoriety.Compute( from, target );
			if ( notor == Notoriety.Innocent && !(from is BaseGuard) )
				return false;
			else
				return true;
		}

		public override bool AllowBenificial( Mobile from, Mobile target )
		{
			int notor = Notoriety.Compute( from, target );
			if ( notor == Notoriety.Criminal || notor == Notoriety.Murderer )
				return false;
			else
				return true;
		}
	}
}