using System;
using Server.Targeting;
using Server.Network;
using Server.Mobiles;
using Server.Items;

namespace Server.SkillHandlers
{
	public class Peacemaking
	{
		public static void Initialize()
		{
			SkillInfo.Table[(int)SkillName.Peacemaking].Callback = new SkillUseCallback( OnUse );
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.RevealingAction();

			m.Target = new InternalMusicTarget();
			m.SendLocalizedMessage( 500617 ); // What instrument shall you play?
			m.RevealingAction();

			return TimeSpan.FromSeconds( 10.0 ); // 10 second delay before beign able to re-use a skill
		}

		private class InternalMusicTarget : Target
		{
			public InternalMusicTarget() : base ( 1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is BaseInstrument )
				{
					from.SendMessage( "Wich creature do you want to pacify?" );
					from.Target = new InternalTarget( (BaseInstrument)targeted );
				}
				else
				{
					from.SendLocalizedMessage( 500619 ); // That is not a musical instrument.
				}
			}
		}

		private class InternalTarget : Target
		{
			private BaseInstrument m_Instrument;

			public InternalTarget( BaseInstrument instrument ) :  base ( 10, false, TargetFlags.None )
			{
				m_Instrument = instrument;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				from.RevealingAction();

				if ( !( targeted is Mobile ) )
				{
					return;
				}
				
				if ( !from.CheckSkill( SkillName.Musicianship, 0.0, 100.0 ) )
				{
					m_Instrument.PlayInstrumentBadly( from );
					from.SendLocalizedMessage( 500612 ); // You play poorly, and there is no effect.
					return;
				}

				// Should each BaseCreature have a "Peacemaking" difficulties?
				if ( !from.CheckSkill( SkillName.Peacemaking, 0.0, 100.0 ) )
				{
					m_Instrument.PlayInstrumentBadly( from );
					from.SendLocalizedMessage( 500613 ); // You attempt to calm everyone, but fail.
					return;
				}

				m_Instrument.PlayInstrumentWell( from );
				from.SendLocalizedMessage( 500615 ); // You play your hypnotic music, stopping the battle.
				
				if ( targeted is BaseCreature )
				{
					BaseCreature creature = (BaseCreature)targeted;
					creature.Pacify( from, DateTime.Now + TimeSpan.FromMinutes( 2 ) );
				}

				else if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					m.SendLocalizedMessage( 500616 ); // You hear lovely music, and forget to continue battling!
					m.Warmode = false;
				}
			}
		}
	}
}