// Magic Trap Spell Script v.1.0 by SickLab

using System;
using Server.Targeting;
using Server.Network;
using Server.Items;

namespace Server.Spells.Second
{
	public class MagicTrapSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Magic Trap", "In Jux",
				SpellCircle.Second,
				212,
				Reagent.Garlic,
				Reagent.SpidersSilk,
				Reagent.SulfurousAsh
			);

		public MagicTrapSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( TrapableContainer item )
		{
			if ( !Caster.CanSee( item ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( item.TrapType != TrapType.None && item.TrapType != TrapType.MagicTrap )
			{
				base.DoFizzle();
			}
			else if ( CheckSequence() )
			{
				SpellHelper.Turn( Caster, item );

				item.TrapType = TrapType.MagicTrap;
				item.TrapPower = 1;

				Point3D loc = item.GetWorldLocation();

				Effects.PlaySound( loc, item.Map, 0x1EF );

				Effects.SendLocationEffect( new Point3D( loc.X + 1, loc.Y, loc.Z ), item.Map, 0x376A, 16 );
				Effects.SendLocationEffect( new Point3D( loc.X - 1, loc.Y, loc.Z ), item.Map, 0x376A, 16 );
				Effects.SendLocationEffect( new Point3D( loc.X, loc.Y + 1, loc.Z ), item.Map, 0x376A, 16 );
				Effects.SendLocationEffect( new Point3D( loc.X, loc.Y - 1, loc.Z ), item.Map, 0x376A, 16 );
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private MagicTrapSpell m_Owner;

			public InternalTarget( MagicTrapSpell owner ) : base( 12, false, TargetFlags.None )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is TrapableContainer )
				{
					m_Owner.Target( (TrapableContainer)o );
				}
				else
				{
					from.SendMessage( "You can't trap that" );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}